#ifndef INCLUDED_TRANSLT_API_H
#define INCLUDED_TRANSLT_API_H

#include "windows.h"
#include "string.h"

#ifndef _T
  #ifdef UNICODE
    #define _T(x) L ## x
  #else
    #define _T(x) x
  #endif
#endif

//Interface for TransLT library version 14.0.0.5

#ifdef WIN64
    const TCHAR *TRLTDLLFILE = _T("..\\..\\..\\..\\Library\\Win64\\TransLT.Win64.dll");
#else
  #ifdef WIN32
    const TCHAR *TRLTDLLFILE = _T("..\\..\\..\\..\\Library\\Win32\\TransLT.Win32.dll");
  #endif
#endif

HINSTANCE hDllImporter;

#define MY_PLACE_INI_TEXT   _T("My Place")
#define MAX_LEN_NAMES       255
#define MAX_LEN_AXES        6
#define MAX_LEN_HTML_COLOR  8
#define MAX_LEN_FOLDER_NAME 32
#define MAX_LEN_ENTITY_INFO 32
#define MAX_LEN_ENTITY_DESC 128

typedef void (__stdcall *TProgressProc) (const int);

typedef enum _EncodingType {
    etANSI, 
    etUTF8, 
    etUnicode, 
    etBigEndianUnicode
} TEncodingType, *PEncodingType;

typedef enum _PlainUnits {
    puKilometer,             // [km]        Kilometer
    puMeter,                 // [m]         Meter
    puDeciMeter,             // [dm]        Decimeter
    puCentiMeter,            // [cm]        Centimeter
    puMilliMeter,            // [mm]        Millimeter
    puIntlNauticalMile,      // [kmi]       International Nautical Mile
    puIntlInch,              // [in]        International Inch
    puIntlFoot,              // [ft]        International Foot
    puIntlYard,              // [yd]        International Yard
    puIntlStatuteMile,       // [mi]        International Statute Mile
    puIntlFathom,            // [fath]      International Fathom
    puIntlChain,             // [ch]        International Chain
    puIntlLink,              // [link]      International Link
    puUSSurvInch,            // [us-in]     U.S. Surveyors Inch
    puUSSurvFoot,            // [us-ft]     U.S. Surveyors Foot
    puUSSurvYard,            // [us-yd]     U.S. Surveyors Yard
    puUSSurvChain,           // [us-ch]     U.S. Surveyors Chain
    puUSSurvStatuteMile,     // [us-mi]     U.S. Surveyors Statute Mile
    puIndianYard,            // [ind-yd]    Indian Yard
    puIndianFoot,            // [ind-ft]    Indian Foot
    puIndianChain,           // [ind-ch]    Indian Chain
	puSurvLink,              //[us-lk]     U.S. Surveyors Link
	puClarkeFoot,            //[clk-ft]    Clarkes Foot
	puClarkeYard,            //[clk-yd]    Clarkes Yard
	puClarkeLink,            //[clk-lk]    Clarkes Link
	puClarkeChain            //[clk-ch]    Clarkes Chain
} TPlainUnits, *PPlainUnits;          

typedef enum _AreaUnits {
    auSqKilometer,           // [km^2]      Square Kilometer
    auSqMeter,               // [m^2]       Square Meter
    auSqDeciMeter,           // [dm^2]      Square Decimeter
    auSqCentiMeter,          // [cm^2]      Square Centimeter
    auSqMilliMeter,          // [mm^2]      Square Millimeter
    auAres,                  // [a]         Ares
    auHectares,              // [ha]        Hectares
    auIntlAcres,             // [ac]        International Acres
    auSqIntlInch,            // [sq.in]     Square International Inch
    auSqIntlFoot,            // [sq.ft]     Square International Foot
    auSqIntlYard,            // [sq.yd]     Square International Yard
    auSqIntlStatuteMile,     // [sq.mi]     Square International Statute Mile
    auUSSurvAcres,           // [us-ac]     U.S. Surveyors Acres
    auSqUSSurvInch,          // [sq.us-in]  Square U.S. Surveyors Inch
    auSqUSSurvFoot,          // [sq.us-ft]  Square U.S. Surveyors Foot
    auSqUSSurvYard,          // [sq.us-yd]  Square U.S. Surveyors Yard
    auSqUSSurvStatuteMile,   // [sq.us-mi]  Square U.S. Surveyors Statute Mile
    auSqIndianYard,          // [sq.ind-yd] Square Indian Yard
    auSqIndianFoot           // [sq.ind-ft] Square Indian Foot
} TAreaUnits, *PAreaUnits;

typedef enum _CRStype {
    ctXYZ,                   //Geocentric (X,Y,Z)
    ctGeo,                   //Geographic (phi,lambda) or (phi,lambda,h)
    ctNEH                    //Projected (N,E) or (N,E,H)
} TCRStype, *PCRStype;

typedef enum _CRSdimension {
    cd2D,                    //Dimension 2D: (phi,lambda) or (N,E) 
    cd3D                     //Dimension 3D: (phi,lambda,h) or (N,E,H)
} TCRSdimension, *PCRSdimension;

typedef wchar_t TCharArrayNames[MAX_LEN_NAMES];
typedef wchar_t TCharArrayAxes[MAX_LEN_AXES];
typedef struct _ModelDescription {
    TCharArrayNames ModelName;               //Model name
    TCharArrayNames TransDescription;        //Model description
    TCharArrayNames CooSysIniName;           //Initial Coordinate System Name
    TCharArrayNames CooSysFinName;           //Final Coordinate System Name
    TCRStype CooSysIniType;                  //Initial Coordinate Reference System Type
    TCRStype CooSysFinType;                  //Final Coordinate System Type
    TCRSdimension CooSysIniDim;              //Initial Coordinate System Dimension
    TCRSdimension CooSysFinDim;              //Final Coordinate System Dimension
    int CodeCRSini;                          //CRS code for initial system from EPSG database
    int CodeCRSfin;                          //CRS code for final system from EPSG database
    TCharArrayAxes AltNameAxisNini;          //Alternative name for the North axis, the initial system
    TCharArrayAxes AltNameAxisEini;          //Alternative name for the East axis, the initial system
    TCharArrayAxes AltNameAxisNfin;          //Alternative name for the North axis, the final system
    TCharArrayAxes AltNameAxisEfin;          //Alternative name for the East axis, the final system
    double TotalAcurracy;                    //Total accuracy of the transformation
    BOOL SupportReverseCalc;                 //If the model supports or not the reverse transformation
    BOOL IsTimeDependent;                    //If the model is time-dependent and if necessary the introduction of the epoch
    BOOL IsUseTimeEpochT0;                   //If the model use time epoch and if necessary the introduction of the epoch T0
} TModelDescription, *PModelDescription;

typedef wchar_t TCharArrayHtmlColor[MAX_LEN_HTML_COLOR];
typedef wchar_t TCharArrayFolderName[MAX_LEN_FOLDER_NAME];
typedef struct _GEDrawOptions {
    TCharArrayHtmlColor PlgLabelsHtmlColor;  //Color and opacity for all polygons labels (Polygon Number)
    TCharArrayHtmlColor PlgLinesHtmlColor;   //Color and opacity for all polygons lines
    BOOL PlgDrawLabels;                      //Draw or not labels for all polygons
    BOOL PlgAddAreaToName;                   //Add area value to entity name for polygons
    TCharArrayHtmlColor PathLabelsHtmlColor; //Color and opacity for all path labels (Start and End texts)
    BOOL PathDrawLabels;                     //Draw or not labels for all paths
    BOOL PathAddLengthToName;                //Add length value to entity name for paths
    int PathIdxSymbIcon;                     //Index of symbol image for start and end points,
                                             //[0..11] see images in TransLT in Option form
    TCharArrayHtmlColor PtsLabelsHtmlColor;  //Color and opacity for all points labels from group
    BOOL AddDescrToName;                     //Add text description to entity name
    double LabelsDistToViewKm;               //Distance from which are seen all labels, necessary when
                                             //used double click on the label in Google Earth.
    TCharArrayFolderName FoldPlg;            //Folder name for polygons
    TCharArrayFolderName FoldPath;           //Folder name for paths
    TCharArrayFolderName FoldGrpPts;         //Folder name for points grup
    double CorPhi;                           //Correction to adjust the latitude  on save
    double CorLam;                           //Correction to adjust the longitude on save
                                             //  The two corrections can be used only if the transformation
                                             //  is less accurate, otherwise the values must be 0.0
} TGEDrawOptions, *PGEDrawOptions;

typedef enum _EntityType {
    etPolygon, 
    etPath, 
    etGrpPts
} TEntityType, *PEntityType;

typedef enum _DrawLabelType {
  dltDefault,  //Draw or not labels is used from PlgDrawLabels from TGEDrawOptions
  dltNotDraw,  //Labels will not be drawn regardless of the general setting from PlgDrawLabels from TGEDrawOptions
  dltDraw      //Labels will be drawn regardless of the general setting from PlgDrawLabels from TGEDrawOptions
} TDrawLabelType, *PDrawLabelType;

typedef wchar_t TCharArrayEntityInfo[MAX_LEN_ENTITY_INFO];
typedef wchar_t TCharArrayEntityDesc[MAX_LEN_ENTITY_DESC];
typedef struct _GEEntityProperties {
    TEntityType EntityType;
    TCharArrayEntityInfo Info1;              //Polygon Number, Start Text for path or Group Name for points
    TCharArrayEntityDesc Info2;              //Polygon Description or End Text for path
    TCharArrayEntityInfo InfoA;              //Area, used only for polygon
    TCharArrayEntityInfo InfoL;              //Perimeter for polygon or Length for path
    BOOL AddAtoName;                         //Add in parentheses the area to the text of the polygon label
    BOOL AddLtoName;                         //Add in parentheses the length to the text of the path label
    TCharArrayEntityDesc InfoAd1;            //Additional information
    TCharArrayEntityDesc InfoAd2;            //Additional information
    TCharArrayEntityDesc InfoAd3;            //Additional information
    TCharArrayEntityDesc InfoAd4;            //Additional information
    TCharArrayEntityDesc InfoAd5;            //Additional information
    TCharArrayEntityDesc InfoAd6;            //Additional information
    int IdxImgSymb;                          //Index of symbol image for points, used only for points group,
                                             //  [0..11] see images in TransLT software (Option form)
    TCharArrayHtmlColor HtmlColor;           //Fill color and opacity for: polygon, path lines or for symbols of points
    TCharArrayHtmlColor HtmlLineColor;       //Fill color and opacity for polygon lines. For empty string is used 
                                             //PlgLinesHtmlColor from TGEDrawOptions
    double LinesWidth;                       //Width of the lines in pixels for polygon or path (Value >= 0.0)
    TDrawLabelType DrawLabel;                //Only for polygons draw or not label or use default draw labels PlgDrawLabels 
    //from TGEDrawOptions
    TCharArrayHtmlColor HtmlLabelColor;      //Fill color and opacity for for polygon labels and group points labels. For  
                                             //empty string is used PlgLabelsHtmlColor from TGEDrawOptions
    double LabelScale;                       //Label scale factor for polygons and points
    double SymbolScale;                      //Symbol scale factor for points
    BOOL UseAddBox;                          //Display information about the entity
    BOOL UseAddInfo1;                        //  Add information about Polygon Number, Start Text or Group Name
    BOOL UseAddInfo2;                        //  Add information about Polygon Description, End Text or Points Group Description
    BOOL UseAddInfoA;                        //  Add information about Polygon Area
    BOOL UseAddInfoL;                        //  Add information about Perimeter or Length for path
    BOOL UseAddAdr;                          //  Add address of the entity
    TCharArrayEntityDesc AddAdr;             //    Text of address
    BOOL UseAddDest;                         //  Add destination of the entity
    TCharArrayEntityDesc AddDest;            //    Text of destination
    BOOL UseAddUsage;                        //  Add usage of the entity
    TCharArrayEntityDesc AddUsage;           //    Text of usage
    BOOL UseAddLnk;                          //  Add URL link of the entity
    TCharArrayEntityDesc AddLinkName;        //    Text of page name
    TCharArrayEntityDesc AddLinkURL;         //    Text of URL link
    TCharArrayFolderName SubFolder;          //Entity subfolder
} TGEEntityProperties, *PGEEntityProperties;

typedef wchar_t TCharArrayLangWords[MAX_LEN_ENTITY_INFO];
typedef struct _GEEntLangWords {             //In english:
    TCharArrayLangWords StrEntMyPlace;       //  "My Place"
    TCharArrayLangWords StrEntPolygonNoName; //  "Polygon - No name"
    TCharArrayLangWords StrEntPathNoName;    //  "Path - No name"
    TCharArrayLangWords StrEntGrpPtsNoName;  //  "Group points - No name"
} TGEEntLangWords, *PGEEntLangWords;

typedef struct _GEKmlLangWords {             //In english:
    TCharArrayLangWords StrKmlPlgNumber;     //  "Number"
    TCharArrayLangWords StrKmlPathStart;     //  "Start"
    TCharArrayLangWords StrKmlPathEnd;       //  "End"
    TCharArrayLangWords StrKmlGrpPtsName;    //  "Group name"
    TCharArrayLangWords StrKmlDescription;   //  "Description"
    TCharArrayLangWords StrKmlArea;          //  "Area"
    TCharArrayLangWords StrKmlPerimeter;     //  "Perimeter"
    TCharArrayLangWords StrKmlLength;        //  "Length"
    TCharArrayLangWords StrKmlInfoAd1;       //  "Additional information"
    TCharArrayLangWords StrKmlInfoAd2;       //  "Additional information"
    TCharArrayLangWords StrKmlInfoAd3;       //  "Additional information"
    TCharArrayLangWords StrKmlInfoAd4;       //  "Additional information"
    TCharArrayLangWords StrKmlInfoAd5;       //  "Additional information"
    TCharArrayLangWords StrKmlInfoAd6;       //  "Additional information"
    TCharArrayLangWords StrKmlInformation;   //  "Information"
    TCharArrayLangWords StrKmlAddress;       //  "Address"
    TCharArrayLangWords StrKmlDestination;   //  "Destination"
    TCharArrayLangWords StrKmlUsage;         //  "Usage"
} TGEKmlLangWords, *PGEKmlLangWords;

typedef struct _AngleAspect {
    BOOL UseDecDeg;   //0.000000000000 or 000'0.000000"
    BOOL UseNSEW;     //Use signs N and S for latitude, E and W for
                      //longitude instead of plus and minus. This setting
                      //is possible only when not using decimal degrees.
    wchar_t ChDeg;    //Character for degree
    wchar_t ChMin;    //Character for minutes
    wchar_t ChSec;    //Character for seconds
    int DecNoSec;     //Decimals for seconds. For decimal degrees
                      //is added 6 to this number.
} TAngleAspect, *PAngleAspect;

//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//SetErrMsgFileName (lpFileName)
typedef BOOL (WINAPI *SETERRMSGFILENAME)(const LPCWSTR);
//GetErrMsgFileName (*lpFileName, *nMaxCount)
typedef void (WINAPI *GETERRMSGFILENAME)(LPWSTR*, int*);

//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//Note: Check if the file contains transformation models, if the models has
//      coordinate operations and the file version.
//ModelFileIsCompatible (lpModelFileName)
typedef BOOL (WINAPI *MODELFILEISCOMPATIBLE)(const LPCWSTR);

//Note: For files with a single model (files with mtr extension) ModelIndex is
//      always 1 and for those with more models (files with mtrs or xml extension) can
//      be 1 up.
//ModelHasCooOperations (lpModelFileName, ModelIndex, *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL (WINAPI *MODELHASCOOOPERATIONS)(const LPCWSTR, const int, int*, LPWSTR*, int*);

//Note: Same note at the function ModelHasCooOperations.
//ModelIsReversible (lpModelFileName, ModelIndex, *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL (WINAPI *MODELISREVERSIBLE)(const LPCWSTR, const int, int*, LPWSTR*, int*);

//Note: - Only models that makes a transformation:
//        1. From projected coordinates to geographic coordinates, (N,E -> phi,lambda);
//        2. From geographic coordinates to projected coordinates for a model
//           that supports reverse transformation, (phi,lambda <- N,E).
//      - Checking the datum must be done by the user. Accepted ellipsoid are
//        GRS80, ETRS89 or WGS84 and prime meridian must be Greenwich.
//ModelIsCompatibleToDrawInGoogleEarth (lpModelFileName, ModelIndex, *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL (WINAPI *MODELISCOMPATIBLETODRAWINGOOGLEEARTH)(const LPCWSTR, const int, int*, LPWSTR*, int*);

//Note: The description is read directly from the file and does not consider the
//      use of the model in reverse.
//GetModelDescriptionFromFile (lpModelFileName, ModelIndex, *ModelDescription, *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL (WINAPI *GETMODELDESCRIPTIONFROMFILE)(const LPCWSTR, const int, TModelDescription*, 
                                                   int*, LPWSTR*, int*);

//GetCountModels (lpModelFileName)
typedef int (WINAPI *GETCOUNTMODELS)(const LPCWSTR);

//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//Note: - Same note at the function ModelHasCooOperations.
//      - If you use reversed model and do not know if the model is reversible
//        or not then you need to test first if the model is reversible or not
//        by using the function ModelIsReversible.
//      - Path for grid files is required only for models that use grid files
//        for coordinates offsets. If the grid files listed inside the model
//        file are defined with full path then in this case is not necessary to
//        introduce the search path. Grid files are searched first in the
//        transformation model directory.
//InitTrPntModel (lpModelFileName, lpGridFilesPath, ModelIndex, PlainUMtype, UseReversed, 
//                *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL (WINAPI *INITTRPNTMODEL)(const LPCWSTR, const LPCWSTR, 
                                      const int, const TPlainUnits, const BOOL, 
                                      int*, LPWSTR*, int*);

//DoneTrPntModel ()
typedef void (WINAPI *DONETRPNTMODEL)(void);

//Note: The description is taken from the model was initialized with function
//      InitTrPntModel. If at the initialization was used UseReversed value when
//      the initial and final coordinate system has been already reversed.
//GetTrPntModelDescription (*ModelDescription)  
typedef BOOL (WINAPI *GETTRPNTMODELDESCRIPTION)(TModelDescription*);

//Set the value of the epoch T0 in decimal year used to calculate the time difference of the dynamic coordinates.
//If the value is omitted then current date is used.
typedef void (WINAPI *SETTRPNTEPOCHT0)(const double);

//Set the value of the epoch T in decimal year of the dynamic coordinates.
//If the value is omitted then current date is used.
typedef void (WINAPI *SETTRPNTEPOCHT)(const double);

//Returns the epoch T0 in dewcimal year used to calculate the time difference of the dynamic coordinates.
typedef BOOL(WINAPI *GETTRPNTEPOCHT0)(double*);

//Returns the epoch T in dewcimal year of the dynamic coordinates.
typedef BOOL (WINAPI *GETTRPNTEPOCHT)(double*);

//AddPoint (lpPntName, XorEorPhi, YorNorLam, ZorHorHe, lpPntCode)  
typedef void (WINAPI *ADDPOINT)(const LPCWSTR, 
                                const double, const double, const double, 
                                const LPCWSTR);

//SetPoint (Index, lpPntName, XorEorPhi, YorNorLam, ZorHorHe, lpPntCode)  
typedef void (WINAPI *SETPOINT)(const int, 
                                const LPCWSTR, 
                                const double, const double, const double, 
                                const LPCWSTR);

//GetPoint (Index, *lpPntName, *nMaxPntNameCount, *XorEorPhi, *YorNorLam, *ZorHorHe, 
//                 *lpPntCode, *nMaxPntCodeCount)  
typedef BOOL (WINAPI *GETPOINT)(const int, LPWSTR*, int*, double*, double*, double*, 
                                LPWSTR*, int*);

//Note: If transformation was not made when the coordinates are formatted
//      according to the initial coordinate system and if the transformation
//      was made when formatting is under final coordinate system.
//GetPointFormated (Index, *lpPntName, *nMaxPntNameCount, 
//                         *lpXorEorPhi, *nMaxXorEorPhiCount, 
//                         *lpYorNorLam, *nMaxYorNorLamCount,
//                         *lpZorHorHe, *nMaxZorHorHeCount,
//                         *lpPntCode, *nMaxPntCodeCount,
//                         AngleAspect, DecNoXYZ, DecNoNEH, DecNoGeoHe)
typedef BOOL (WINAPI *GETPOINTFORMATED)(int, LPWSTR*, int*, LPWSTR*, int*, LPWSTR*, int*, 
                                        LPWSTR*, int*, LPWSTR*, int*, 
                                        const PAngleAspect, const int, const int, const int);

//DeletePoint (Index)  
typedef void (WINAPI *DELETEPOINT)(int);

//GetCountPoints ()  
typedef int (WINAPI *GETCOUNTPOINTS)(void);

//Note: - Axis inversion is possible only in the case of geographic or projected
//        coordinates.
//      - For geographical coordinates normal order is phi, lambda and for projected
//        coordinates is N, E.
//LoadPointsFromFile (lpFileName, InvertAxes12, AngleAspect, PlainUMtype, ChCsv, *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL(WINAPI *LOADPOINTSFROMFILE)(const LPCWSTR, const BOOL, const PAngleAspect, const TPlainUnits, 
	                                     const wchar_t, int*, LPWSTR*, int*);

//Note: - Axis inversion is possible only in the case of geographic or projected
//        coordinates.
//      - For geographical coordinates normal order is phi, lambda and for projected
//        coordinates is N, E.
//SavePointsToFile (lpFileName, FixCoo, InvertAxes12, AngleAspect, PlainUMtype, ChCsv, DecNoXYZ, DecNoNEH, 
//                  DecNoGeoHe, EncodingType)
typedef BOOL(WINAPI *SAVEPOINTSTOFILE)(const LPCWSTR, const BOOL, const BOOL, const PAngleAspect, const TPlainUnits,
                                       const wchar_t, const int, const int, const int, const TEncodingType);

//TransformPoints (ProgressProc, lpStrCooOp, *ErrNo, *lpErrTxt, *nMaxCount)
typedef BOOL (WINAPI *TRANSFORMPOINTS)(TProgressProc, const LPCWSTR, int*, LPWSTR*, int*);

//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//Note: - Same note at the function ModelHasCooOperations.
//      - Same note at the function InitTrPntModel
//      - In the case of models which are used for drawing in Google Earth is
//        testing if the model is compatible or not for this command. The model
//        is compatible if it can make make a transformation from projected
//        coordinate system to geographical coordinate system: (N,E -> phi,lambda) or
//        reversed (phi,lambda <- N,E). Same note at the function
//        ModelIsCompatibleToDrawInGoogleEarth.
//InitTrGEModel (lpModelFileName, lpGridFilesPath, ModelIndex, PlainUMtype, UseReversed, GEEntLangWords,
//               *ErrNo, *lpErrTxt, *nMaxCount)  
typedef BOOL (WINAPI *INITTRGEMODEL)(const LPCWSTR, const LPCWSTR, const int, const TPlainUnits, const BOOL, 
                                     const PGEEntLangWords, int*, LPWSTR*, int*);

//DoneTrGEModel ()
typedef void (WINAPI *DONETRGEMODEL)(void);

//Note: The description is taken from the model was initialized with function
//      InitTrPntModel. If at the initialization was used UseReversed value when
//      the initial and final coordinate system has been already reversed.
//GetTrGEModelDescription (*ModelDescription)  
typedef BOOL (WINAPI *GETTRGEMODELDESCRIPTION)(TModelDescription*);

//Set the value of the epoch T0 in decimal year used to calculate the time difference of the dynamic coordinates.
//If the value is omitted then current date is used.
typedef void (WINAPI *SETTRGEEPOCHT0)(const double);

//Set the value of the epoch T in decimal year of the dynamic coordinates.
//If the value is omitted then current date is used.
typedef void (WINAPI *SETTRGEEPOCHT)(const double);

//Returns the epoch T0 in dewcimal year used to calculate the time difference of the dynamic coordinates.
typedef BOOL(WINAPI *GETTRGEEPOCHT0)(double*);

//Returns the epoch T in dewcimal year of the dynamic coordinates.
typedef BOOL (WINAPI *GETTRGEEPOCHT)(double*);

//AddGEEntity (GEEntityProperties)  
typedef void (WINAPI *ADDGEENTITY)(const PGEEntityProperties);

//SetGEEntity (EntIndex, GEEntityProperties)  
typedef void (WINAPI *SETGEENTITY)(const int, const PGEEntityProperties);

//DeleteGEEntity (EntIndex)  
typedef void (WINAPI *DELETEGEENTITY)(const int);

//GetGEEntity (EntIndex, *GEEntityProperties)  
typedef BOOL (WINAPI *GETGEENTITY)(const int, TGEEntityProperties*);

//ClearGEEntities ()  
typedef void (WINAPI *CLEARGEENTITIES)(void);

//GetCountGEEntites ()  
typedef int (WINAPI *GETCOUNTGEENTITES)(void);

//AddPointToGEEntity (EntIndex, lpPntName, EorPhi, NorLam)  
typedef void (WINAPI *ADDPOINTTOGEENTITY)(const int, const LPCWSTR, 
                                          const double, const double);

//SetPointToGEEntity (EntIndex, PntIndex, lpPntName, EorPhi, NorLam)  
typedef void (WINAPI *SETPOINTTOGEENTITY)(const int, const int, const LPCWSTR, 
                                          const double, const double);

//GetPointFromGEEntity (EntIndex, PntIndex, *lpPntName, *nMaxPntNameCount, *EorPhi, *NorLam)  
typedef BOOL (WINAPI *GETPOINTFROMGEENTITY)(const int, const int, LPWSTR*, int*, 
                                            double*, double*);

//GetPointFromGEEntityFormated (EntIndex, PntIndex, 
//                              *lpPntName, *nMaxPntNameCount, 
//                              *lpEorPhi, *nMaxEorPhiCount, 
//                              *lpNorLam, *nMaxNorLamCount,
//                              CoordIsGeog, AngleAspect, DecNoNEH)  
typedef BOOL (WINAPI *GETPOINTFROMGEENTITYFORMATED)(const int, const int, 
                                                    LPWSTR*, int*, LPWSTR*, int*, LPWSTR*, int*, 
                                                    const BOOL, const PAngleAspect, const int);

//GetCountPointsOfGEEntity (EntIndex)  
typedef int (WINAPI *GETCOUNTPOINTSOFGEENTITY)(const int);

//DeleteAllPointsOfGEEntity (EntIndex)  
typedef void (WINAPI *DELETEALLPOINTSOFGEENTITY)(const int);

//Note: This calculation is made only when the coordinates of entities are
//      projected, in geographical coordinates this calculation is wrong.
//      Checking this must be done by the user.
//CalculateAreaAndLengthToGEEntity (EntIndex, AreaUMtype, AreaDecNo, LengthUMtype, LengthDecNo)
typedef void (WINAPI *CALCULATEAREAANDLENGTHTOGEENTITY)(const int, const TAreaUnits, const int, 
                                                        const TPlainUnits, const int);

//Note: The format of csv file must comply the TransLT file format (file with
//      entities inside).
//LoadGEEntitiesFromFile (lpFileName, InvertAxes12, AngleAspect, ChCsv, *ErrNo, *lpErrTxt, *nMaxCount)
typedef BOOL (WINAPI *LOADGEENTITIESFROMFILE)(const LPCWSTR, const BOOL, const PAngleAspect, const wchar_t, 
                                                 int*, LPWSTR*, int*);

//Note: Saving data can be made for three types of files: csv (TransLT format
//      with entities), kml (Uncompressed Google Earth format) or kmz
//      (Compressed Google Earth format). The file format is chosen by its
//      extension.
//SaveGEEntitiesToFile (lpFileName, lpProgName, lpProgVer, lpWebPage, lpPlaceName, InvertAxes12,
//                      AngleAspect, ChCsv, GEDrawOptions, GEKmlLangWords, EncodingType, 
//                      *ErrNo, *lpErrTxt, *nMaxCountr)
typedef BOOL (WINAPI *SAVEGEENTITIESTOFILE)(const LPCWSTR, const LPCWSTR, const LPCWSTR, const LPCWSTR, 
                                            const LPCWSTR, const BOOL, const PAngleAspect, const wchar_t, 
                                            const PGEDrawOptions, const PGEKmlLangWords, const TEncodingType,
                                            int*, LPWSTR*, int*);

//Note: This command only works if Google Earth is installed on your computer,
//      otherwise an error occurs.
//DrawGEEntitiesToGoogleEarth (lpProgName, lpProgVer, lpWebPage, lpPlaceName, InvertAxes12,
//                             GEDrawOptions, GEKmlLangWords,  
//                            *ErrNo, *lpErrTxt, *nMaxCountr)
typedef BOOL (WINAPI *DRAWGEENTITIESTOGOOGLEEARTH)(const LPCWSTR, const LPCWSTR, const LPCWSTR, const LPCWSTR,  
                                                   const LPCWSTR, const PGEDrawOptions, const PGEKmlLangWords,
                                                   int*, LPWSTR*, int*);

//Note: Save in file the information for which Google Earth will zoom on the
//      coordinates of a rectangle. Extension of the file can be only kml.
//SaveGEZoomToFile (lpFileName, lpFolderName, PhiMin, LamMin, PhiMax, LamMax)
typedef BOOL(WINAPI *SAVEGEZOOMTOFILE)(const LPCWSTR, const LPCWSTR, 
                                       const double, const double, const double, const double);

//Note: This command only works if Google Earth is installed on your computer,
//      otherwise an error occurs.
//DrawGEZoomToGoogleEarth (lpFileName, lpFolderName, lPhiMin, LamMin, PhiMax, LamMax,
//                         *ErrNo, *lpErrTxt, *nMaxCountr)
typedef BOOL(WINAPI *DRAWGEZOOMTOGOOGLEEARTH)(const LPCWSTR, const LPCWSTR, 
                                              const double, const double, const double, const double,
                                              int*, LPWSTR*, int*);

//Note: Delete all temporary files used for drawing entities in Google Earth.
//ClearTempFilesGE ()  
typedef void (WINAPI* CLEARTEMPFILESGE)(void);

//Note: To make this transformation of the points of entities points must be
//      entered as projected coordinates (N,E). Conversion should be made from
//      projected coordinates to geographic coordinates.
//TransformPointsOfAllGEEntities (ProgressProc, lpStrCooOp, *ErrNo, *lpErrTxt, *nMaxCount)
typedef BOOL (WINAPI *TRANSFORMPOINTSOFALLGEENTITIES)(TProgressProc, const LPCWSTR, int*, LPWSTR*, int*);

#endif
